/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.table;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.opengion.fukurou.util.ApplicationInfo;
import org.opengion.fukurou.util.ErrorMessage;
import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.db.DBTableModel;
import org.opengion.hayabusa.db.TableFilter;
import org.opengion.hayabusa.resource.ResourceManager;

/**
 * <p>AbstractTableFilter は、TableUpda インターフェースを継承した、DBTableModel 処理用の
 * Abstract実装クラスです。<br />
 *
 *
 * @version  0.9.0  2000/10/17
 * @author   Kazuhiko Hasegawa
 * @since    JDK1.1,
 */
abstract class AbstractTableFilter implements TableFilter {
	protected static final String CR	= HybsSystem.CR ;	// 5.1.1.0 (2009/12/01) CR 定義をここで行う。

	protected DBTableModel	table		= null;
	protected String		modifyType	= null;
	protected int[]			rowNo		= null;
	protected boolean		isDebug		= false;
	protected ApplicationInfo appInfo	= null;	// 3.8.7.0 (2006/12/15)
	protected String		sql			= null; // 4.2.4.0 (2008/06/23)
	protected String		dbid		= null; // 4.2.4.0 (2008/06/23)
	protected ResourceManager resource	= null; // 4.3.7.4 (2009/07/01)

	private int 			errCode		= ErrorMessage.OK;
	private ErrorMessage	errMessage	= null;

	private final Map<String,String>	keysVals	= new HashMap<String,String>();

	/**
	 * DBTableModel をセットします。 <br />
	 *
	 * @param	table DBTableModel
	 */
	public void setDBTableModel( final DBTableModel table ) {
		this.table = table;
	}

	/**
	 * データ処理の方法(A:追加 C:更新 D:削除)を指定します。
	 *
	 * 通常は、DBTableModel に自動設定されている modifyType を元に、データ処理方法を
	 * 選別します。(A:追加 C:更新 D:削除)
	 * この場合、行単位で modifyType の値を取得して判別する必要がありますが、一般には
	 * 処理対象は、全件おなじ modifyType である可能性が高いです。
	 * また、selectedAll などで強制的に全件処理対象とする場合は、modifyType に値が
	 * 設定さていません。その様な場合に外部より modifyType を指定します。
	 * デフォルトは、自動判定 です。</p>
	 *
	 * @param  type データ処理の方法(A:追加 C:更新 D:削除)
	 */
	public void setModifyType( final String type ) {
		modifyType = type;
	}

	/**
	 * キーと値のペアの変数配列を受け取ります。
	 *
	 * @param   keys String[]
	 * @param   vals String[]
	 */
	public void setKeysVals( final String[] keys,final String[] vals ) {
		if( keys != null && vals != null ) {
			for( int i=0; i<keys.length; i++ ) {
				keysVals.put( keys[i].toUpperCase(Locale.JAPAN),vals[i] );
			}
		}
	}

	/**
	 * 表示データの HybsSystem.ROW_SELECTED_KEY を元に、選ばれた 行を
	 * 処理の対象とします。
	 *
	 * @param   rowNoTmp int[]
	 */
	public void setParameterRows( final int[] rowNoTmp ) {
		if( rowNoTmp != null ) {
			int size = rowNoTmp.length ;
			rowNo = new int[size];
			System.arraycopy( rowNoTmp,0,rowNo,0,size );
		}
	}

	/**
	 * アクセスログ取得の為,ApplicationInfo オブジェクトを設定します。 <br />
	 *
	 * @og.rev 3.8.7.0 (2006/12/15) 新規追加
	 *
	 * @param   appInfo ApplicationInfo
	 */
	public void setApplicationInfo( final ApplicationInfo appInfo ) {
		this.appInfo = appInfo;
	}
	
	/**
	 * DBIDを指定します。<br />
	 * 
	 * @og.rev 4.2.4.0 (2008/06/23) 新規追加
	 * 
	 * @param dbid DBID
	 */
	public void setDbid( final String dbid ) {
		this.dbid = dbid;
	}
	
	/**
	 * ボディー部分のSQLを指定します。<br />
	 * 
	 * @og.rev 4.2.4.0 (2008/06/23) 新規追加
	 * 
	 * @param sql ボディー部分のSQL
	 */
	public void setSql( final String sql ) {
		this.sql = sql;
	}

	/**
	 * リソースオブジェクトを指定します。<br />
	 * 
	 * @og.rev 4.3.7.4 (2009/07/01) 新規追加
	 * 
	 * @param resource リソースオブジェクト
	 */
	public void setResource( final ResourceManager resource ) {
		this.resource = resource;
	}

	/**
	 * デバッグ情報を表示するかどうかを指定します。
	 * true でデバッグ情報を表示します。
	 *
	 * @param   flag boolean
	 */
	public void setDebug( final boolean flag ) {
		isDebug = flag;
	}

	/**
	 * エラーコード を取得します。 <br />
	 * エラーコード は、ErrorMessage クラスで規定されているコードです。
	 *
	 * @return   エラーコード
	 */
	public int getErrorCode() {
		return errCode;
	}

	/**
	 * エラーメッセージオブジェクト を取得します。 <br />
	 *
	 * @return   エラーメッセージオブジェクト
	 */
	public ErrorMessage getErrorMessage() {
		return errMessage;
	}

	/**
	 * タイトルとエラーコードを指定して、エラーメッセージオブジェクト を作成します。 <br />
	 * すでに、作成済みの場合は、作成済みのオブジェクトを、まだ、未作成の場合は、
	 * 新規に作成します。
	 *
	 * @param title String
	 * @param code  int
	 * @return   エラーメッセージオブジェクト
	 */
	protected ErrorMessage makeErrorMessage( final String title,final int code ) {
		if( errMessage == null ) {
			errMessage = new ErrorMessage( title );
		}
		if( errCode < code ) { errCode = code; }
		return errMessage;
	}

	/**
	 *  カラム名配列(String[])より、対応するカラムNo配列(int[])を作成します。
	 *
	 * @param	nameArray カラム名配列
	 * @return	カラムNo配列
	 */
	protected int[] getTableColumnNo( final String[] nameArray ) {
		int[] clmNo = new int[ nameArray.length ];
		for( int i=0; i<clmNo.length; i++ ) {
			clmNo[i] = table.getColumnNo( nameArray[i] );
		}
		return clmNo;
	}

	/**
	 *  設定されたパラメータキーに対する値を取得します。
	 * 引数、および、パラメータが null の場合は、 null を返します。
	 *
	 * @param	key String パラメータキー
	 * @return	パラメータ値
	 */
	protected String getValue( final String key ) {
		return keysVals.get( key );
	}
}
